#include "stdafx.h"
#include "ComiCmcSdk.h"
#include "ComiEcatSdk_Api.h"
#include <MMSystem.h>

CComiCmc::CComiCmc()
{
	TEcSlvProdInfo ProdInfo;
	int nErr;

	memset(&CmcInfo, 0, sizeof(CmcInfo));

	int nSlvNum = ecNet_ScanSlaves(NETID, NULL);
	for(int i=1; i<nSlvNum; i++)	{
		ecSlv_GetProdInfo_A(NETID, i, &ProdInfo, NULL);
		if(ProdInfo.ProdCode == ECAT_CMC) {
			CmcInfo.Index[CmcInfo.DevNum] = i;
			CmcInfo.Address[CmcInfo.DevNum] = ecSlv_SlvIdx2PhysAddr(NETID, i, &nErr);
			CmcInfo.DevNum++;
		}
	}	

	for(int i=0; i<CmcInfo.DevNum; i++) {
		for(int j=0; j<6; j++) {
			TCmc_AxisOutPDO *pOutPDO = GetOutPDO(CmcInfo.Address[i], j);
			pOutPDO->CtlWord = 6;
		}
	}
}

BOOL IsTimeOut(ULONG dwStartT_ms, ULONG dwTimeOutVal_ms)
{
	ULONG dwElpsTime;
	ULONG dwCurTime = timeGetTime();
	if(dwCurTime < dwStartT_ms){
		if(dwCurTime > dwTimeOutVal_ms){
			return true;
		}else{
			dwElpsTime = dwCurTime + (ULONG)((ULONG)0xffffffff - dwStartT_ms) + 1;
		}
	}else{
		dwElpsTime = dwCurTime - dwStartT_ms;
	}

	if(dwElpsTime > dwTimeOutVal_ms)
		return true;
	else
		return false;
}

TCmc_AxisInPDO* CComiCmc::GetInPDO(int nAxis)
{
	if(CmcInfo.DevNum == 0)
		return NULL;
	t_byte *pInPDO = (t_byte *)ecSlv_InPDO_GetBufPtr(NETID, CmcInfo.Address[nAxis/6], 0, NULL);
	pInPDO = pInPDO + (sizeof(TCmc_AxisInPDO) * (nAxis%6));

	return (TCmc_AxisInPDO *)pInPDO;
}

TCmc_AxisInPDO* CComiCmc::GetInPDO(int nPhysAddr, int nSubIdx)
{
	if(CmcInfo.DevNum == 0)
		return NULL;
	t_byte *pInPDO = (t_byte *)ecSlv_InPDO_GetBufPtr(NETID, nPhysAddr, 0, NULL);
	pInPDO = pInPDO + (sizeof(TCmc_AxisInPDO) * nSubIdx);

	return (TCmc_AxisInPDO *)pInPDO;
}

TCmc_AxisOutPDO* CComiCmc::GetOutPDO(int nAxis)
{
	if(CmcInfo.DevNum == 0)
		return NULL;
	t_byte *pOutPDO = (t_byte *)ecSlv_OutPDO_GetBufPtr(NETID, CmcInfo.Address[nAxis/6], 0, NULL);
	pOutPDO = pOutPDO + (sizeof(TCmc_AxisOutPDO) * (nAxis%6));

	return (TCmc_AxisOutPDO *)pOutPDO;
}

TCmc_AxisOutPDO* CComiCmc::GetOutPDO(int nPhysAddr, int nSubIdx)
{
	if(CmcInfo.DevNum == 0)
		return NULL;
	t_byte *pOutPDO = (t_byte *)ecSlv_OutPDO_GetBufPtr(NETID, nPhysAddr, 0, NULL);
	pOutPDO = pOutPDO + (sizeof(TCmc_AxisOutPDO) * nSubIdx);

	return (TCmc_AxisOutPDO *)pOutPDO;
}

USHORT* CComiCmc::GetDiPDO(int nPhysAddr)
{
	if(CmcInfo.DevNum == 0)
		return NULL;
	t_byte *pInPDO = (t_byte *)ecSlv_InPDO_GetBufPtr(NETID, nPhysAddr, 0, NULL);
	pInPDO = pInPDO + (sizeof(TCmc_AxisInPDO) * 6);

	return (USHORT*)pInPDO;
}

USHORT* CComiCmc::GetDoPDO(int nPhysAddr)
{
	if(CmcInfo.DevNum == 0)
		return NULL;
	t_byte *pOutPDO = (t_byte *)ecSlv_OutPDO_GetBufPtr(NETID, nPhysAddr, 0, NULL);
	pOutPDO = pOutPDO + (sizeof(TCmc_AxisOutPDO) * 6);

	return (USHORT*)pOutPDO;
}

LONG CComiCmc::GetNumAxis()
{
	return CmcInfo.DevNum*6;
}

USHORT CComiCmc::GetSlaveAddress(int nAxis)
{
	return CmcInfo.Address[nAxis/6];
}

USHORT CComiCmc::GetSlaveIndex(int nAxis)
{
	return CmcInfo.Index[nAxis/6];
}

BOOL CComiCmc::SetSvon(int nAxis, BOOL SvonState, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;	
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	TCmc_AxisInPDO *pInPDO = GetInPDO(nAxis);
	if(pInPDO == NULL){
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_INPDO_PTR;
		return false;
	}

	TCmc_AxisOutPDO *pOutPDO = GetOutPDO(nAxis);
	if(pOutPDO == NULL){
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_OUTPDO_PTR;
		return false;
	}

	USHORT wOldCtlWord = pOutPDO->CtlWord;
	if(SvonState == 0){
		pOutPDO->CtlWord = (wOldCtlWord & 0xfff0) | 0x0007;
	}else{
		pOutPDO->CtlWord = (wOldCtlWord & 0xfff0) | 0x000f;		
	}	
	
	ecSetBit(pOutPDO->CtlWord, 8, 1); // Halt
	
	return true;
}

BOOL CComiCmc::SetSvon(int nPhysAddr, int nSubIdx, BOOL SvonState, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;	
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	TCmc_AxisOutPDO *pOutPDO = GetOutPDO(nPhysAddr, nSubIdx);
	if(pOutPDO == NULL){
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_OUTPDO_PTR;
		return false;
	}

	USHORT wOldCtlWord = pOutPDO->CtlWord;
	if(SvonState == 0){
		pOutPDO->CtlWord = (wOldCtlWord & 0xfff0) | 0x0007;
	}else{
		pOutPDO->CtlWord = (wOldCtlWord & 0xfff0) | 0x000f;		
	}	
	
	ecSetBit(pOutPDO->CtlWord, 8, 1); // Halt
	
	return true;
}

BOOL CComiCmc::GetSvon(int nAxis, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	TCmc_AxisOutPDO *pOutPDO = GetOutPDO(nAxis);
	if(pOutPDO == NULL){
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_OUTPDO_PTR;
		return false;
	}

	return ecGetBit(pOutPDO->CtlWord, 3);	
}

BOOL CComiCmc::GetSvon(int nPhysAddr, int nSubIdx, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	TCmc_AxisOutPDO *pOutPDO = GetOutPDO(nPhysAddr, nSubIdx);
	if(pOutPDO == NULL){
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_OUTPDO_PTR;
		return false;
	}

	return ecGetBit(pOutPDO->CtlWord, 3);	
}

BOOL CComiCmc::MoveStart(int nAxis, int Distance, int *pErrCode)
{
	TCmc_AxisOutPDO *pOutPDO = GetOutPDO(nAxis);
	if(pOutPDO == NULL){
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_OUTPDO_PTR;
		return false;
	}

	// OP-MODE "Profile Position Mode"  üũ //
	TCmc_AxisInPDO *pInPDO = GetInPDO(nAxis);
	ULONG dwStartT = timeGetTime();
	while(pInPDO && pInPDO->OpMode != 1){
		pOutPDO->OpMode = 1;
		Sleep(1);
		if(IsTimeOut(dwStartT, 1000)){
			if(pErrCode) *pErrCode = CMC_ERR_OPMODE_TIMEOUT;
			return false;
		}
	}	
	
	ecSetBit(pOutPDO->CtlWord, 8, 0);

	pOutPDO->TargPos = Distance;

	ecSetBit(pOutPDO->CtlWord, 6, 1); // Relative

	if(ecGetBit(pOutPDO->CtlWord, 4)){
		ecSetBit(pOutPDO->CtlWord, 4, 0);
		Sleep(PDO_INTERVAL);
	}

	ecSetBit(pOutPDO->CtlWord, 4, 1);
	Sleep(PDO_INTERVAL);
	ecSetBit(pOutPDO->CtlWord, 4, 0);
	Sleep(PDO_INTERVAL);
	return true;
}

BOOL CComiCmc::MoveStart(int nPhysAddr, int nSubIdx, int Distance, int *pErrCode)
{
	TCmc_AxisOutPDO *pOutPDO = GetOutPDO(nPhysAddr, nSubIdx);
	if(pOutPDO == NULL){
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_OUTPDO_PTR;
		return false;
	}

	// OP-MODE "Profile Position Mode"  üũ //
	TCmc_AxisInPDO *pInPDO = GetInPDO(nPhysAddr, nSubIdx);
	ULONG dwStartT = timeGetTime();
	while(pInPDO && pInPDO->OpMode != 1){
		pOutPDO->OpMode = 1;
		Sleep(1);
		if(IsTimeOut(dwStartT, 1000)){
			if(pErrCode) *pErrCode = CMC_ERR_OPMODE_TIMEOUT;
			return false;
		}
	}	

	ecSetBit(pOutPDO->CtlWord, 8, 0);
	
	pOutPDO->TargPos = Distance;

	ecSetBit(pOutPDO->CtlWord, 6, 1); // Relative

	if(ecGetBit(pOutPDO->CtlWord, 4)){
		ecSetBit(pOutPDO->CtlWord, 4, 0);
		Sleep(PDO_INTERVAL);
	}

	ecSetBit(pOutPDO->CtlWord, 4, 1);
	Sleep(PDO_INTERVAL);
	ecSetBit(pOutPDO->CtlWord, 4, 0);
	Sleep(PDO_INTERVAL);
	return true;
}

BOOL CComiCmc::MoveToStart(int nAxis, int Position, int *pErrCode)
{
	TCmc_AxisOutPDO *pOutPDO = GetOutPDO(nAxis);
	if(pOutPDO == NULL){
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_OUTPDO_PTR;
		return false;
	}

	TCmc_AxisInPDO *pInPDO = GetInPDO(nAxis);
	ULONG dwStartT = timeGetTime();
	while(pInPDO && pInPDO->OpMode != 1){
		pOutPDO->OpMode = 1;
		Sleep(1);
		if(IsTimeOut(dwStartT, 1000)){
			if(pErrCode) *pErrCode = CMC_ERR_OPMODE_TIMEOUT;
			return false;
		}
	}
	
	ecSetBit(pOutPDO->CtlWord, 8, 0);

	pOutPDO->TargPos = Position;

	ecSetBit(pOutPDO->CtlWord, 6, 0); // Absolute

	// Move Start Trigger //
	if(ecGetBit(pOutPDO->CtlWord, 4)){
		ecSetBit(pOutPDO->CtlWord, 4, 0);
		Sleep(PDO_INTERVAL);
	}
	ecSetBit(pOutPDO->CtlWord, 4, 1);
	Sleep(PDO_INTERVAL);
	ecSetBit(pOutPDO->CtlWord, 4, 0);
	Sleep(PDO_INTERVAL);
	return true;
}

BOOL CComiCmc::MoveToStart(int nPhysAddr, int nSubIdx, int Position, int *pErrCode)
{
	TCmc_AxisOutPDO *pOutPDO = GetOutPDO(nPhysAddr, nSubIdx);
	if(pOutPDO == NULL){
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_OUTPDO_PTR;
		return false;
	}

	TCmc_AxisInPDO *pInPDO = GetInPDO(nPhysAddr, nSubIdx);
	ULONG dwStartT = timeGetTime();
	while(pInPDO && pInPDO->OpMode != 1){
		pOutPDO->OpMode = 1;
		Sleep(1);
		if(IsTimeOut(dwStartT, 1000)){
			if(pErrCode) *pErrCode = CMC_ERR_OPMODE_TIMEOUT;
			return false;
		}
	}

	ecSetBit(pOutPDO->CtlWord, 8, 0);
	
	pOutPDO->TargPos = Position;

	ecSetBit(pOutPDO->CtlWord, 6, 0); // Absolute

	// Move Start Trigger //
	if(ecGetBit(pOutPDO->CtlWord, 4)){
		ecSetBit(pOutPDO->CtlWord, 4, 0);
		Sleep(PDO_INTERVAL);
	}
	ecSetBit(pOutPDO->CtlWord, 4, 1);
	Sleep(PDO_INTERVAL);
	ecSetBit(pOutPDO->CtlWord, 4, 0);
	Sleep(PDO_INTERVAL);
	return true;
}

BOOL CComiCmc::VMoveStart(int nAxis, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;

	TCmc_AxisOutPDO *pOutPDO = GetOutPDO(nAxis);
	if(pOutPDO == NULL){
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_OUTPDO_PTR;
		return false;
	}

	TCmc_AxisInPDO *pInPDO = GetInPDO(nAxis);
	ULONG dwStartT = timeGetTime();
	while(pInPDO && pInPDO->OpMode != 3){
		pOutPDO->OpMode = 3;
		Sleep(1);
		if(IsTimeOut(dwStartT, 1000)){
			if(pErrCode) *pErrCode = CMC_ERR_OPMODE_TIMEOUT;
			return false;
		}
	}
	
	ecSetBit(pOutPDO->CtlWord, 8, 0);

	return true;
}

BOOL CComiCmc::VMoveStart(int nPhysAddr, int nSubIdx, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;

	TCmc_AxisOutPDO *pOutPDO = GetOutPDO(nPhysAddr, nSubIdx);
	if(pOutPDO == NULL){
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_OUTPDO_PTR;
		return false;
	}

	TCmc_AxisInPDO *pInPDO = GetInPDO(nPhysAddr, nSubIdx);
	ULONG dwStartT = timeGetTime();
	while(pInPDO && pInPDO->OpMode != 3){
		pOutPDO->OpMode = 3;
		Sleep(1);
		if(IsTimeOut(dwStartT, 1000)){
			if(pErrCode) *pErrCode = CMC_ERR_OPMODE_TIMEOUT;
			return false;
		}
	}
	
	ecSetBit(pOutPDO->CtlWord, 8, 0);

	return true;
}

BOOL CComiCmc::HomeMoveStart(int nAxis, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;

	t_i32 TmpErrCode = 0;

	TCmc_AxisOutPDO *pOutPDO = GetOutPDO(nAxis);
	if(pOutPDO == NULL){
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_OUTPDO_PTR;
		return false;
	}

	// OP-MODE "Homing Mode"  üũ //
	TCmc_AxisInPDO *pInPDO = GetInPDO(nAxis);
	ULONG dwStartT = timeGetTime();
	while(pInPDO && pInPDO->OpMode != 6){ // "Homing Mode" ƴϸ "Homing Mode" ȯ
		pOutPDO->OpMode = 6;
		Sleep(1);
		if(IsTimeOut(dwStartT, 1000)){
			if(pErrCode) *pErrCode = CMC_ERR_OPMODE_TIMEOUT;
			return false;
		}
	}

	ecSetBit(pOutPDO->CtlWord, 8, 0);	

	// Move Start Trigger //
	if(ecGetBit(pOutPDO->CtlWord, 4)){
		ecSetBit(pOutPDO->CtlWord, 4, 0);
		Sleep(PDO_INTERVAL);
	}
	ecSetBit(pOutPDO->CtlWord, 4, 1); // Homing operation start
	Sleep(PDO_INTERVAL);
	ecSetBit(pOutPDO->CtlWord, 4, 0);
	Sleep(PDO_INTERVAL);
	return true;
}

BOOL CComiCmc::HomeMoveStart(int nPhysAddr, int nSubIdx, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;

	t_i32 TmpErrCode = 0;

	TCmc_AxisOutPDO *pOutPDO = GetOutPDO(nPhysAddr, nSubIdx);
	if(pOutPDO == NULL){
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_OUTPDO_PTR;
		return false;
	}

	// OP-MODE "Homing Mode"  üũ //
	TCmc_AxisInPDO *pInPDO = GetInPDO(nPhysAddr, nSubIdx);
	ULONG dwStartT = timeGetTime();
	while(pInPDO && pInPDO->OpMode != 6){ // "Homing Mode" ƴϸ "Homing Mode" ȯ
		pOutPDO->OpMode = 6;
		Sleep(1);
		if(IsTimeOut(dwStartT, 1000)){
			if(pErrCode) *pErrCode = CMC_ERR_OPMODE_TIMEOUT;
			return false;
		}
	}	

	ecSetBit(pOutPDO->CtlWord, 8, 0);
	
	// Move Start Trigger //
	if(ecGetBit(pOutPDO->CtlWord, 4)){
		ecSetBit(pOutPDO->CtlWord, 4, 0);
		Sleep(PDO_INTERVAL);
	}
	ecSetBit(pOutPDO->CtlWord, 4, 1); // Homing operation start
	Sleep(PDO_INTERVAL);
	ecSetBit(pOutPDO->CtlWord, 4, 0);
	Sleep(PDO_INTERVAL);
	return true;
}

BOOL CComiCmc::Stop(int nAxis, int *pErrCode)
{
	TCmc_AxisOutPDO *pOutPDO = GetOutPDO(nAxis);
	if(pOutPDO == NULL){
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_OUTPDO_PTR;
		return false;
	}

	int nData = 0;
	int ErrCode = 0;

	// 0x605D Halt Option Code
	ecSlv_ReadCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x605D+((nAxis%6)*0x800), 0, false, 2, &nData, &ErrCode);
	if( nData != 1 ) //  ƴϸ
	{
		nData = 1;
		ecSlv_WriteCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x605D+((nAxis%6)*0x800), 0, false, 2, &nData, &ErrCode);
	}

	ecSetBit(pOutPDO->CtlWord, 8, 1); // Halt
	
	return true;
}

BOOL CComiCmc::Stop(int nPhysAddr, int nSubIdx, int *pErrCode)
{
	TCmc_AxisOutPDO *pOutPDO = GetOutPDO(nPhysAddr, nSubIdx);
	if(pOutPDO == NULL){
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_OUTPDO_PTR;
		return false;
	}

	int nData = 0;
	int ErrCode = 0;

	// 0x605D Halt Option Code
	ecSlv_ReadCoeSdo(NETID, nPhysAddr, 0x605D+(nSubIdx*0x800), 0, false, 2, &nData, &ErrCode);
	if( nData != 1 ) //  ƴϸ
	{
		nData = 1;
		ecSlv_WriteCoeSdo(NETID, nPhysAddr, 0x605D+(nSubIdx*0x800), 0, false, 2, &nData, &ErrCode);
	}

	ecSetBit(pOutPDO->CtlWord, 8, 1); // Halt
	
	return true;
}

BOOL CComiCmc::EmgStop(int nAxis, int *pErrCode)
{
	TCmc_AxisOutPDO *pOutPDO = GetOutPDO(nAxis);
	if(pOutPDO == NULL){
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_OUTPDO_PTR;
		return false;
	}

	int nData = 0;
	int ErrCode = 0;

	// 0x605A Quick Stop Option Code
	ecSlv_ReadCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x605A+((nAxis%6)*0x800), 0, false, 2, &nData, &ErrCode);
	if( nData != 2 ) //  ƴϸ
	{
		nData = 2;
		ecSlv_WriteCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x605A+((nAxis%6)*0x800), 0, false, 2, &nData, &ErrCode);
	}

	ecSetBit(pOutPDO->CtlWord, 3, 1); // Servo OFF
	
	return true;
}

BOOL CComiCmc::EmgStop(int nPhysAddr, int nSubIdx, int *pErrCode)
{
	TCmc_AxisOutPDO *pOutPDO = GetOutPDO(nPhysAddr, nSubIdx);
	if(pOutPDO == NULL){
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_OUTPDO_PTR;
		return false;
	}

	int nData = 0;
	int ErrCode = 0;

	// 0x605A Quick Stop Option Code
	ecSlv_ReadCoeSdo(NETID, nPhysAddr, 0x605A+(nSubIdx*0x800), 0, false, 2, &nData, &ErrCode);
	if( nData != 2 ) //  ƴϸ
	{
		nData = 2;
		ecSlv_WriteCoeSdo(NETID, nPhysAddr, 0x605A+(nSubIdx*0x800), 0, false, 2, &nData, &ErrCode);
	}

	ecSetBit(pOutPDO->CtlWord, 3, 1); // Servo OFF
	
	return true;
}

BOOL CComiCmc::SetSpeedEnv(int nAxis, int WorkSpeed, int Accel, int Decel, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	ecSlv_WriteCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x6081+((nAxis%6)*0x800), 0, false, 4, &WorkSpeed, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	ecSlv_WriteCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x6083+((nAxis%6)*0x800), 0, false, 4, &Accel, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	ecSlv_WriteCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x6084+((nAxis%6)*0x800), 0, false, 4, &Decel, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	return true;
}

BOOL CComiCmc::SetSpeedEnv(int nPhysAddr, int nSubIdx, int WorkSpeed, int Accel, int Decel, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	ecSlv_WriteCoeSdo(NETID, nPhysAddr, 0x6081+(nSubIdx*0x800), 0, false, 4, &WorkSpeed, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	ecSlv_WriteCoeSdo(NETID, nPhysAddr, 0x6083+(nSubIdx*0x800), 0, false, 4, &Accel, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	ecSlv_WriteCoeSdo(NETID, nPhysAddr, 0x6084+(nSubIdx*0x800), 0, false, 4, &Decel, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	return true;
}

BOOL CComiCmc::GetSpeedEnv(int nAxis, int *pWorkSpeed, int *pAccel, int *pDecel, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	ecSlv_ReadCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x6081+((nAxis%6)*0x800), 0, false, 4, pWorkSpeed, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	ecSlv_ReadCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x6083+((nAxis%6)*0x800), 0, false, 4, pAccel, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	ecSlv_ReadCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x6084+((nAxis%6)*0x800), 0, false, 4, pDecel, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	return true;
}

BOOL CComiCmc::GetSpeedEnv(int nPhysAddr, int nSubIdx, int *pWorkSpeed, int *pAccel, int *pDecel, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	ecSlv_ReadCoeSdo(NETID, nPhysAddr, 0x6081+(nSubIdx*0x800), 0, false, 4, pWorkSpeed, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	ecSlv_ReadCoeSdo(NETID, nPhysAddr, 0x6083+(nSubIdx*0x800), 0, false, 4, pAccel, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	ecSlv_ReadCoeSdo(NETID, nPhysAddr, 0x6084+(nSubIdx*0x800), 0, false, 4, pDecel, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	return true;
}

BOOL CComiCmc::SetMaxSpeed(int nAxis, int MaxSpeed, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	ecSlv_WriteCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x607F+((nAxis%6)*0x800), 0, false, 4, &MaxSpeed, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	return true;
}

BOOL CComiCmc::SetMaxSpeed(int nPhysAddr, int nSubIdx, int MaxSpeed, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	ecSlv_WriteCoeSdo(NETID, nPhysAddr, 0x607F+(nSubIdx*0x800), 0, false, 4, &MaxSpeed, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	return true;
}

BOOL CComiCmc::GetMaxSpeed(int nAxis, int *pMaxSpeed, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	ecSlv_ReadCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x607F+((nAxis%6)*0x800), 0, false, 4, pMaxSpeed, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	return true;
}

BOOL CComiCmc::GetMaxSpeed(int nPhysAddr, int nSubIdx, int *pMaxSpeed, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	ecSlv_ReadCoeSdo(NETID, nPhysAddr, 0x607F+(nSubIdx*0x800), 0, false, 4, pMaxSpeed, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	return true;
}

BOOL CComiCmc::SetTargetVelocity(int nAxis, int Velocity, int *pErrCode)
{
	TCmc_AxisOutPDO *pOutPDO = GetOutPDO(nAxis);
	if(pOutPDO == NULL){
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_OUTPDO_PTR;
		return false;
	}

	pOutPDO->TargVel = Velocity;

	return true;
}

BOOL CComiCmc::SetTargetVelocity(int nPhysAddr, int nSubIdx, int Velocity, int *pErrCode)
{
	TCmc_AxisOutPDO *pOutPDO = GetOutPDO(nPhysAddr, nSubIdx);
	if(pOutPDO == NULL){
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_OUTPDO_PTR;
		return false;
	}

	pOutPDO->TargVel = Velocity;

	return true;
}

BOOL CComiCmc::GetTargetVelocity(int nAxis, int *pVelocity, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	ecSlv_ReadCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x60FF+((nAxis%6)*0x800), 0, false, 4, pVelocity, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	return true;
}

BOOL CComiCmc::GetTargetVelocity(int nPhysAddr, int nSubIdx, int *pVelocity, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	ecSlv_ReadCoeSdo(NETID, nPhysAddr, 0x60FF+(nSubIdx*0x800), 0, false, 4, pVelocity, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	return true;
}

BOOL CComiCmc::SetHomeMode(int nAxis, int HomeMethod, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	ecSlv_WriteCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x6098+((nAxis%6)*0x800), 0, false, 2, &HomeMethod, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	return true;
}

BOOL CComiCmc::SetHomeMode(int nPhysAddr, int nSubIdx, int HomeMethod, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	ecSlv_WriteCoeSdo(NETID, nPhysAddr, 0x6098+(nSubIdx*0x800), 0, false, 2, &HomeMethod, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	return true;
}

BOOL CComiCmc::GetHomeMode(int nAxis, int *pHomeMethod, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	t_i32 nData = 0;
	ecSlv_ReadCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x6098+((nAxis%6)*0x800), 0, false, 2, &nData, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	memcpy(pHomeMethod, &nData, sizeof(nData));
	return true;
}

BOOL CComiCmc::GetHomeMode(int nPhysAddr, int nSubIdx, int *pHomeMethod, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	t_i32 nData = 0;
	ecSlv_ReadCoeSdo(NETID, nPhysAddr, 0x6098+(nSubIdx*0x800), 0, false, 2, &nData, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	memcpy(pHomeMethod, &nData, sizeof(nData));
	return true;
}

BOOL CComiCmc::SetHomeSpeedEnv(int nAxis, int WorkSpeed, int StopSpeed, int AccDec, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	ecSlv_WriteCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x6099+((nAxis%6)*0x800), 1, false, 4, &WorkSpeed, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	ecSlv_WriteCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x6099+((nAxis%6)*0x800), 2, false, 4, &StopSpeed, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	ecSlv_WriteCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x609A+((nAxis%6)*0x800), 0, false, 4, &AccDec, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	return true;
}

BOOL CComiCmc::SetHomeSpeedEnv(int nPhysAddr, int nSubIdx, int WorkSpeed, int StopSpeed, int AccDec, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	ecSlv_WriteCoeSdo(NETID, nPhysAddr, 0x6099+(nSubIdx*0x800), 1, false, 4, &WorkSpeed, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	ecSlv_WriteCoeSdo(NETID, nPhysAddr, 0x6099+(nSubIdx*0x800), 2, false, 4, &StopSpeed, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	ecSlv_WriteCoeSdo(NETID, nPhysAddr, 0x609A+(nSubIdx*0x800), 0, false, 4, &AccDec, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	return true;
}

BOOL CComiCmc::GetHomeSpeedEnv(int nAxis, int *pWorkSpeed, int *pStopSpeed, int *pAccDec, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	ecSlv_ReadCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x6099+((nAxis%6)*0x800), 1, false, 4, pWorkSpeed, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	ecSlv_ReadCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x6099+((nAxis%6)*0x800), 2, false, 4, pStopSpeed, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	ecSlv_ReadCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x609A+((nAxis%6)*0x800), 0, false, 4, pAccDec, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	return true;
}

BOOL CComiCmc::GetHomeSpeedEnv(int nPhysAddr, int nSubIdx, int *pWorkSpeed, int *pStopSpeed, int *pAccDec, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	ecSlv_ReadCoeSdo(NETID, nPhysAddr, 0x6099+(nSubIdx*0x800), 1, false, 4, pWorkSpeed, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	ecSlv_ReadCoeSdo(NETID, nPhysAddr, 0x6099+(nSubIdx*0x800), 2, false, 4, pStopSpeed, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	ecSlv_ReadCoeSdo(NETID, nPhysAddr, 0x609A+(nSubIdx*0x800), 0, false, 4, pAccDec, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	return true;
}

BOOL CComiCmc::StResetPosition(int nAxis, int *pErrCode)
{
	TCmc_AxisOutPDO *pOutPDO = GetOutPDO(nAxis);
	if(pOutPDO == NULL){
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_OUTPDO_PTR;
		return false;
	}

	ecSetBit(pOutPDO->CtlWord, 11, 1); // Position Reset
	Sleep(PDO_INTERVAL);
	ecSetBit(pOutPDO->CtlWord, 11, 0);
	
	return true;
}

BOOL CComiCmc::StResetPosition(int nPhysAddr, int nSubIdx, int *pErrCode)
{
	TCmc_AxisOutPDO *pOutPDO = GetOutPDO(nPhysAddr, nSubIdx);
	if(pOutPDO == NULL){
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_OUTPDO_PTR;
		return false;
	}

	ecSetBit(pOutPDO->CtlWord, 11, 1); // Position Reset
	Sleep(PDO_INTERVAL);
	ecSetBit(pOutPDO->CtlWord, 11, 0);

	return true;
}


BOOL CComiCmc::StGetPosition(int nAxis, int TargCntr, int *pPosition, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	if(TargCntr) 
		ecSlv_ReadCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x6064+((nAxis%6)*0x800), 0, false, 4, pPosition, &TmpErrCode);
	else 
		ecSlv_ReadCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x6062+((nAxis%6)*0x800), 0, false, 4, pPosition, &TmpErrCode);

	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}

	return true;
}

BOOL CComiCmc::StGetPosition(int nPhysAddr, int nSubIdx, int TargCntr, int *pPosition, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	if(TargCntr) 
		ecSlv_ReadCoeSdo(NETID, nPhysAddr, 0x6064+(nSubIdx*0x800), 0, false, 4, pPosition, &TmpErrCode);
	else 
		ecSlv_ReadCoeSdo(NETID, nPhysAddr, 0x6062+(nSubIdx*0x800), 0, false, 4, pPosition, &TmpErrCode);

	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}

	return true;
}

BOOL CComiCmc::StGetCurSpeed(int nAxis, int TargCntr, int *pCurSpeed, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	if(TargCntr) 
		ecSlv_ReadCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x606C+((nAxis%6)*0x800), 0, false, 4, pCurSpeed, &TmpErrCode);
	else 
		ecSlv_ReadCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x606B+((nAxis%6)*0x800), 0, false, 4, pCurSpeed, &TmpErrCode);

	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}

	return true;
}

BOOL CComiCmc::StGetCurSpeed(int nPhysAddr, int nSubIdx, int TargCntr, int *pCurSpeed, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	if(TargCntr) 
		ecSlv_ReadCoeSdo(NETID, nPhysAddr, 0x606C+(nSubIdx*0x800), 0, false, 4, pCurSpeed, &TmpErrCode);
	else 
		ecSlv_ReadCoeSdo(NETID, nPhysAddr, 0x606B+(nSubIdx*0x800), 0, false, 4, pCurSpeed, &TmpErrCode);

	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}

	return true;
}

BOOL CComiCmc::StGetMotFlag(int nAxis, void *pMotFlag, int *pErrCode)
{
	TCmc_AxisInPDO *pInPDO = GetInPDO(nAxis);
	if(pInPDO == NULL){
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_INPDO_PTR;
		return false;
	}

	memcpy(pMotFlag, &pInPDO->StsWord, sizeof(pInPDO->StsWord));

	return true;
}

BOOL CComiCmc::StGetMotFlag(int nPhysAddr, int nSubIdx, void *pMotFlag, int *pErrCode)
{
	TCmc_AxisInPDO *pInPDO = GetInPDO(nPhysAddr, nSubIdx);
	if(pInPDO == NULL){
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_INPDO_PTR;
		return false;
	}

	memcpy(pMotFlag, &pInPDO->StsWord, sizeof(pInPDO->StsWord));

	return true;
}

BOOL CComiCmc::SetPolarity(int nAxis, int Polarity, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	ecSlv_WriteCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x607E+((nAxis%6)*0x800), 0, false, 1, &Polarity, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	return true;
}

BOOL CComiCmc::SetPolarity(int nPhysAddr, int nSubIdx, int Polarity, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	ecSlv_WriteCoeSdo(NETID, nPhysAddr, 0x607E+(nSubIdx*0x800), 0, false, 1, &Polarity, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	return true;
}

BOOL CComiCmc::GetPolarity(int nAxis, int *pPolarity, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	t_i32 nData = 0;
	ecSlv_ReadCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x607E+((nAxis%6)*0x800), 0, false, 1, &nData, &TmpErrCode);	
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	memcpy(pPolarity, &nData, sizeof(nData));
	return true;
}

BOOL CComiCmc::GetPolarity(int nPhysAddr, int nSubIdx, int *pPolarity, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	t_i32 nData = 0;
	ecSlv_ReadCoeSdo(NETID, nPhysAddr, 0x607E+(nSubIdx*0x800), 0, false, 1, &nData, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	memcpy(pPolarity, &nData, sizeof(nData));
	return true;
}

BOOL CComiCmc::SetSoftLimit(int nAxis, int TargLimit, int PosLimit, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	if(TargLimit) 
		ecSlv_WriteCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x607D+((nAxis%6)*0x800), 2, false, 4, &PosLimit, &TmpErrCode);
	else 
		ecSlv_WriteCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x607D+((nAxis%6)*0x800), 1, false, 4, &PosLimit, &TmpErrCode);

	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}

	return true;
}

BOOL CComiCmc::SetSoftLimit(int nPhysAddr, int nSubIdx, int TargLimit, int PosLimit, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	if(TargLimit) 
		ecSlv_WriteCoeSdo(NETID, nPhysAddr, 0x607D+(nSubIdx*0x800), 2, false, 4, &PosLimit, &TmpErrCode);
	else 
		ecSlv_WriteCoeSdo(NETID, nPhysAddr, 0x607D+(nSubIdx*0x800), 1, false, 4, &PosLimit, &TmpErrCode);

	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}

	return true;
}

BOOL CComiCmc::GetSoftLimit(int nAxis, int TargLimit, int *pPosLimit, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	if(TargLimit) 
		ecSlv_ReadCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x607D+((nAxis%6)*0x800), 2, false, 4, pPosLimit, &TmpErrCode);
	else 
		ecSlv_ReadCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x607D+((nAxis%6)*0x800), 1, false, 4, pPosLimit, &TmpErrCode);

	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}

	return true;
}

BOOL CComiCmc::GetSoftLimit(int nPhysAddr, int nSubIdx, int TargLimit, int *pPosLimit, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	if(TargLimit) 
		ecSlv_ReadCoeSdo(NETID, nPhysAddr, 0x607D+(nSubIdx*0x800), 2, false, 4, pPosLimit, &TmpErrCode);
	else 
		ecSlv_ReadCoeSdo(NETID, nPhysAddr, 0x607D+(nSubIdx*0x800), 1, false, 4, pPosLimit, &TmpErrCode);

	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}

	return true;
}

BOOL CComiCmc::ReadMotState(int nAxis, int *pState, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	t_i32 nData = 0;
	ecSlv_ReadCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x2022, (nAxis%6)+1, false, 1, &nData, &TmpErrCode);

	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	memcpy(pState, &nData, sizeof(nData));
	return true;
}

BOOL CComiCmc::ReadMotState(int nPhysAddr, int nSubIdx, int *pState, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	t_i32 nData = 0;
	ecSlv_ReadCoeSdo(NETID, nPhysAddr, 0x2022, nSubIdx+1, false, 1, &nData, &TmpErrCode);

	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	memcpy(pState, &nData, sizeof(nData));
	return true;
}

BOOL CComiCmc::SetWorkCurrent(int nAxis, int TargCurrent, int CurrValue, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	if(TargCurrent) 
		ecSlv_WriteCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x2011, (nAxis%6)+1, false, 2, &CurrValue, &TmpErrCode);
	else 
		ecSlv_WriteCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x2010, (nAxis%6)+1, false, 2, &CurrValue, &TmpErrCode);

	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}

	return true;
}

BOOL CComiCmc::SetWorkCurrent(int nPhysAddr, int nSubIdx, int TargCurrent, int CurrValue, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	if(TargCurrent) 
		ecSlv_WriteCoeSdo(NETID, nPhysAddr, 0x2011, nSubIdx+1, false, 2, &CurrValue, &TmpErrCode);
	else 
		ecSlv_WriteCoeSdo(NETID, nPhysAddr, 0x2010, nSubIdx+1, false, 2, &CurrValue, &TmpErrCode);

	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}

	return true;
}

BOOL CComiCmc::GetWorkCurrent(int nAxis, int TargCurrent, int *pCurrValue, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}	

	t_i32 TmpErrCode = 0;
	t_i32 nData = 0;
	if(TargCurrent) 
		ecSlv_ReadCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x2011, (nAxis%6)+1, false, 2, &nData, &TmpErrCode);
	else 		
		ecSlv_ReadCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x2010, (nAxis%6)+1, false, 2, &nData, &TmpErrCode);

	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	memcpy(pCurrValue, &nData, sizeof(nData));
	return true;
}

BOOL CComiCmc::GetWorkCurrent(int nPhysAddr, int nSubIdx, int TargCurrent, int *pCurrValue, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	t_i32 nData = 0;
	if(TargCurrent) 
		ecSlv_ReadCoeSdo(NETID, nPhysAddr, 0x2011, nSubIdx+1, false, 2, &nData, &TmpErrCode);
	else 
		ecSlv_ReadCoeSdo(NETID, nPhysAddr, 0x2010, nSubIdx+1, false, 2, &nData, &TmpErrCode);

	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	memcpy(pCurrValue, &nData, sizeof(nData));
	return true;
}

BOOL CComiCmc::SetStopCurrent(int nAxis, int CurrValue, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	ecSlv_WriteCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x2012, (nAxis%6)+1, false, 2, &CurrValue, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}

	return true;
}

BOOL CComiCmc::SetStopCurrent(int nPhysAddr, int nSubIdx, int CurrValue, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	ecSlv_WriteCoeSdo(NETID, nPhysAddr, 0x2012, nSubIdx+1, false, 2, &CurrValue, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}

	return true;
}

BOOL CComiCmc::GetStopCurrent(int nAxis, int *pCurrValue, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	t_i32 nData = 0;
	ecSlv_ReadCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x2012, (nAxis%6)+1, false, 2, &nData, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	memcpy(pCurrValue, &nData, sizeof(nData));
	return true;
}

BOOL CComiCmc::GetStopCurrent(int nPhysAddr, int nSubIdx, int *pCurrValue, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	t_i32 nData = 0;
	ecSlv_ReadCoeSdo(NETID, nPhysAddr, 0x2012, nSubIdx+1, false, 2, &nData, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	memcpy(pCurrValue, &nData, sizeof(nData));
	return true;
}

BOOL CComiCmc::SetStepDivision(int nAxis, int StepDiv, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	ecSlv_WriteCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x2015, (nAxis%6)+1, false, 1, &StepDiv, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}

	return true;
}

BOOL CComiCmc::SetStepDivision(int nPhysAddr, int nSubIdx, int StepDiv, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	ecSlv_WriteCoeSdo(NETID, nPhysAddr, 0x2015, nSubIdx+1, false, 1, &StepDiv, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}

	return true;
}

BOOL CComiCmc::GetStepDivision(int nAxis, int *pStepDiv, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	t_i32 nData = 0;
	ecSlv_ReadCoeSdo(NETID, CmcInfo.Address[nAxis/6], 0x2015, (nAxis%6)+1, false, 1, &nData, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	memcpy(pStepDiv, &nData, sizeof(nData));
	return true;
}

BOOL CComiCmc::GetStepDivision(int nPhysAddr, int nSubIdx, int *pStepDiv, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	t_i32 nData = 0;
	ecSlv_ReadCoeSdo(NETID, nPhysAddr, 0x2015, nSubIdx+1, false, 1, &nData, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	memcpy(pStepDiv, &nData, sizeof(nData));
	return true;
}

BOOL CComiCmc::SetSensorChan(int nPhysAddr, int nSubIdx, int nSensor, int SelectChan, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	ecSlv_WriteCoeSdo(NETID, nPhysAddr, 0x2100+nSubIdx, nSensor+1, false, 2, &SelectChan, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}

	return true;
}

BOOL CComiCmc::GetSensorChan(int nPhysAddr, int nSubIdx, int nSensor, int *pConnectChan, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	t_i32 nData = 0;
	ecSlv_ReadCoeSdo(NETID, nPhysAddr, 0x2100+nSubIdx, nSensor+1, false, 2, &nData, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	memcpy(pConnectChan, &nData, sizeof(nData));
	return true;
}

BOOL CComiCmc::SetSensorLogic(int nPhysAddr, int nSubIdx, int nSensor, int Logic, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	ecSlv_WriteCoeSdo(NETID, nPhysAddr, 0x2110+nSubIdx, nSensor+1, false, 1, &Logic, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}

	return true;
}

BOOL CComiCmc::GetSensorLogic(int nPhysAddr, int nSubIdx, int nSensor, int *pLogic, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	t_i32 nData = 0;
	ecSlv_ReadCoeSdo(NETID, nPhysAddr, 0x2110+nSubIdx, nSensor+1, false, 1, &nData, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	memcpy(pLogic, &nData, sizeof(nData));
	return true;
}

BOOL CComiCmc::ReadEncoderCnt(int nChannel, int TargCntr, int *pCountValue, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	t_i32 nData = 0;
	if(TargCntr) 
		ecSlv_ReadCoeSdo(NETID, CmcInfo.Address[nChannel/3], 0x2031, (nChannel%3)+1, false, 2, &nData, &TmpErrCode);
	else 
		ecSlv_ReadCoeSdo(NETID, CmcInfo.Address[nChannel/3], 0x2030, (nChannel%3)+1, false, 4, &nData, &TmpErrCode);

	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	memcpy(pCountValue, &nData, sizeof(nData));
	return true;
}

BOOL CComiCmc::ReadEncoderCnt(int nPhysAddr, int nSubIdx, int TargCntr, int *pCountValue, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_i32 TmpErrCode = 0;
	t_i32 nData = 0;
	if(TargCntr) 
		ecSlv_ReadCoeSdo(NETID, nPhysAddr, 0x2031, nSubIdx+1, false, 2, &nData, &TmpErrCode);
	else 
		ecSlv_ReadCoeSdo(NETID, nPhysAddr, 0x2030, nSubIdx+1, false, 4, &nData, &TmpErrCode);

	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}
	memcpy(pCountValue, &nData, sizeof(nData));
	return true;
}

BOOL CComiCmc::SetCntClear(int nChannel, int TargCntr, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_ui8 nData = 0;
	t_i32 TmpErrCode = 0;
	if(TargCntr) 
		return false;
	else {
		ecSlv_WriteCoeSdo(NETID, CmcInfo.Address[nChannel/3], 0x2032, (nChannel%3)+1, false, 1, &nData, &TmpErrCode);
		nData = 1;
		ecSlv_WriteCoeSdo(NETID, CmcInfo.Address[nChannel/3], 0x2032, (nChannel%3)+1, false, 1, &nData, &TmpErrCode);
	}
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}

	return true;
}

BOOL CComiCmc::SetCntClear(int nPhysAddr, int nSubIdx, int TargCntr, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(CmcInfo.DevNum == 0) {
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_SLAVE_ADDR;
		return false;
	}

	t_ui8 nData = 0;
	t_i32 TmpErrCode = 0;
	if(TargCntr) 
		return false;
	else {
		ecSlv_WriteCoeSdo(NETID, nPhysAddr, 0x2032, nSubIdx+1, false, 1, &nData, &TmpErrCode);
		nData = 1;
		ecSlv_WriteCoeSdo(NETID, nPhysAddr, 0x2032, nSubIdx+1, false, 1, &nData, &TmpErrCode);
	}
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}

	return true;
}



USHORT CComiCmc::DiGetNumChannel()
{
	return CmcInfo.DevNum*16;
}

USHORT CComiCmc::DoGetNumChannel()
{
	return CmcInfo.DevNum*12;
}

BOOL CComiCmc::DiGetOne(int DiChannel, int *pErrCode)
{
	USHORT* pInPDO = GetDiPDO(CmcInfo.Address[DiChannel/16]);
	if(pInPDO == NULL){
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_INPDO_PTR;
		return false;
	}

	return ecGetBit(*pInPDO, DiChannel%16);
}

BOOL CComiCmc::DiGetOne(int nPhysAddr, int DiChannel, int *pErrCode)
{
	USHORT* pInPDO = GetDiPDO(nPhysAddr);
	if(pInPDO == NULL){
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_INPDO_PTR;
		return false;
	}

	return ecGetBit(*pInPDO, DiChannel);
}

USHORT CComiCmc::DiGetMulti(int IniChannel, int NumChannel, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(!(16 - (IniChannel%16) >= NumChannel)) {
		if(pErrCode)	*pErrCode = CMC_ERR_INVALID_PARAMETER;
		return false;
	}

	USHORT* pInPDO = GetDiPDO(CmcInfo.Address[IniChannel/16]);
	if(pInPDO == NULL){
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_INPDO_PTR;
		return false;
	}

	USHORT TempData = 0;
	for(int i=0; i<NumChannel; i++) {
		TempData += (ecGetBit(*pInPDO, (IniChannel%16)+i) << i);
	}

	return TempData;
}

USHORT CComiCmc::DiGetMulti(int nPhysAddr, int *pErrCode)
{
	USHORT* pInPDO = GetDiPDO(nPhysAddr);
	if(pInPDO == NULL){
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_INPDO_PTR;
		return false;
	}

	USHORT TempData = 0;
	for(int i=0; i<16; i++) {
		TempData += (ecGetBit(*pInPDO, i) << i);
	}

	return TempData;
}

BOOL CComiCmc::DoGetOne(int DoChannel, int *pErrCode)
{
	USHORT* pOutPDO = GetDoPDO(CmcInfo.Address[DoChannel/12]);
	if(pOutPDO == NULL){
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_OUTPDO_PTR;
		return false;
	}

	return ecGetBit(*pOutPDO, DoChannel%12);
}

BOOL CComiCmc::DoGetOne(int nPhysAddr, int DoChannel, int *pErrCode)
{
	USHORT* pOutPDO = GetDoPDO(nPhysAddr);
	if(pOutPDO == NULL){
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_OUTPDO_PTR;
		return false;
	}

	return ecGetBit(*pOutPDO, DoChannel);
}

USHORT CComiCmc::DoGetMulti(int IniChannel, int NumChannel, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(!(12 - (IniChannel%12) >= NumChannel)) {
		if(pErrCode)	*pErrCode = CMC_ERR_INVALID_PARAMETER;
		return false;
	}

	USHORT* pOutPDO = GetDoPDO(CmcInfo.Address[IniChannel/12]);
	if(pOutPDO == NULL){
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_OUTPDO_PTR;
		return false;
	}

	USHORT TempData = 0;
	for(int i=0; i<NumChannel; i++) {
		TempData += (ecGetBit(*pOutPDO, (IniChannel%12)+i) << i);
	}

	return TempData;
}

USHORT CComiCmc::DoGetMulti(int nPhysAddr, int *pErrCode)
{
	USHORT* pOutPDO = GetDoPDO(nPhysAddr);
	if(pOutPDO == NULL){
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_OUTPDO_PTR;
		return false;
	}

	USHORT TempData = 0;
	for(int i=0; i<12; i++) {
		TempData += (ecGetBit(*pOutPDO, i) << i);
	}

	return TempData;
}

BOOL CComiCmc::DoPutOne(int DoChannel, BOOL OutState, int *pErrCode)
{
	USHORT* pOutPDO = GetDoPDO(CmcInfo.Address[DoChannel/12]);
	if(pOutPDO == NULL){
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_OUTPDO_PTR;
		return false;
	}

	ecSetBit(*pOutPDO, (DoChannel%12), OutState);
	Sleep(PDO_INTERVAL);

	return true;
}

BOOL CComiCmc::DoPutOne(int nPhysAddr, int DoChannel, BOOL OutState, int *pErrCode)
{
	USHORT* pOutPDO = GetDoPDO(nPhysAddr);
	if(pOutPDO == NULL){
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_OUTPDO_PTR;
		return false;
	}

	ecSetBit(*pOutPDO, DoChannel, OutState);
	Sleep(PDO_INTERVAL);

	return true;
}

BOOL CComiCmc::DoPutMulti(int IniChannel, int NumChannel, USHORT OutState, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;
	if(!(12 - (IniChannel%12) >= NumChannel)) {
		if(pErrCode)	*pErrCode = CMC_ERR_INVALID_PARAMETER;
		return false;
	}

	USHORT* pOutPDO = GetDoPDO(CmcInfo.Address[IniChannel/12]);
	if(pOutPDO == NULL){
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_OUTPDO_PTR;
		return false;
	}

	for(int i=0; i<NumChannel; i++)
		ecSetBit(*pOutPDO, (IniChannel%12)+i, (OutState>>i)&0x1);
	Sleep(PDO_INTERVAL);

	return true;
}

BOOL CComiCmc::DoPutMulti(int nPhysAddr, USHORT OutState, int *pErrCode)
{
	USHORT* pOutPDO = GetDoPDO(nPhysAddr);
	if(pOutPDO == NULL){
		if(pErrCode) *pErrCode = CMC_ERR_INVALID_OUTPDO_PTR;
		return false;
	}

	for(int i=0; i<12; i++)
		ecSetBit(*pOutPDO, i, (OutState>>i)&0x1);
	Sleep(PDO_INTERVAL);

	return true;
}

BOOL CComiCmc::ReadFwVersion(int nSlaveIdx, char *pVersion, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;	

	t_i32 TmpErrCode = 0;
	ecSlv_ReadCoeSdo_A(NETID, nSlaveIdx, 0x100A, 0, false, 8, pVersion, &TmpErrCode);
	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}

	return true;
}

BOOL CComiCmc::SetSettingSave(int nSlaveIdx, int *pErrCode)
{
	if(pErrCode) *pErrCode = 0;

	t_ui16 nData = 0;
	t_i32 TmpErrCode = 0;

	ecSlv_WriteCoeSdo_A(NETID, nSlaveIdx, 0x20F0, 0, false, 2, &nData, &TmpErrCode);
	nData = 1;
	ecSlv_WriteCoeSdo_A(NETID, nSlaveIdx, 0x20F0, 0, false, 2, &nData, &TmpErrCode);

	if(TmpErrCode != 0) {
		if(pErrCode) *pErrCode = TmpErrCode;
		return false;
	}

	return true;
}