
#include "ComiEcatSdk_Def.h"

#define NETID			0
#define ECAT_CMC		0x50323766
#define PDO_INTERVAL	5

#pragma pack(push, 1)

typedef struct{
	t_ui16	StsWord;
	t_i32	ActPos;
	t_i32	ActVel;
	t_byte	OpMode;
}TCmc_AxisInPDO;

typedef struct{
	t_ui16	CtlWord;
	t_i32	TargPos;
	t_i32	TargVel;
	t_byte	OpMode;
}TCmc_AxisOutPDO;

typedef struct{
	t_ui16	DiValue;
}TCmc_DiInPDO;

typedef struct{
	t_ui16	DevNum;
	t_i16	Index[10];
	t_i16	Address[10];
}TCmc_DeviceInfo;

class CComiCmc
{
private:
	TCmc_DeviceInfo		CmcInfo;
public:
	CComiCmc();

	LONG		GetNumAxis();
	USHORT		GetSlaveAddress(int nAxis);
	USHORT		GetSlaveIndex(int nAxis);	
	BOOL		SetSvon(int nAxis, BOOL SvonState, int *pErrCode);
	BOOL		SetSvon(int nPhysAddr, int nSubIdx, BOOL SvonState, int *pErrCode);
	BOOL		GetSvon(int nAxis, int *pErrCode);
	BOOL		GetSvon(int nPhysAddr, int nSubIdx, int *pErrCode);
	BOOL		MoveStart(int nAxis, int Distance, int *pErrCode);
	BOOL		MoveStart(int nPhysAddr, int nSubIdx, int Distance, int *pErrCode);
	BOOL		MoveToStart(int nAxis, int Position, int *pErrCode);
	BOOL		MoveToStart(int nPhysAddr, int nSubIdx, int Position, int *pErrCode);
	BOOL		VMoveStart(int nAxis, int *pErrCode);
	BOOL		VMoveStart(int nPhysAddr, int nSubIdx, int *pErrCode);
	BOOL		HomeMoveStart(int nAxis, int *pErrCode);
	BOOL		HomeMoveStart(int nPhysAddr, int nSubIdx, int *pErrCode);
	BOOL		Stop(int nAxis, int *pErrCode);
	BOOL		Stop(int nPhysAddr, int nSubIdx, int *pErrCode);
	BOOL		EmgStop(int nAxis, int *pErrCode);
	BOOL		EmgStop(int nPhysAddr, int nSubIdx, int *pErrCode);
	BOOL		SetSpeedEnv(int nAxis, int WorkSpeed, int Accel, int Decel, int *pErrCode);
	BOOL		SetSpeedEnv(int nPhysAddr, int nSubIdx, int WorkSpeed, int Accel, int Decel, int *pErrCode);
	BOOL		GetSpeedEnv(int nAxis, int *pWorkSpeed, int *pAccel, int *pDecel, int *pErrCode);
	BOOL		GetSpeedEnv(int nPhysAddr, int nSubIdx, int *pWorkSpeed, int *pAccel, int *pDecel, int *pErrCode);
	BOOL		SetMaxSpeed(int nAxis, int MaxSpeed, int *pErrCode);
	BOOL		SetMaxSpeed(int nPhysAddr, int nSubIdx, int MaxSpeed, int *pErrCode);
	BOOL		GetMaxSpeed(int nAxis, int *pMaxSpeed, int *pErrCode);
	BOOL		GetMaxSpeed(int nPhysAddr, int nSubIdx, int *pMaxSpeed, int *pErrCode);
	BOOL		SetTargetVelocity(int nAxis, int Velocity, int *pErrCode);
	BOOL		SetTargetVelocity(int nPhysAddr, int nSubIdx, int Velocity, int *pErrCode);
	BOOL		GetTargetVelocity(int nAxis, int *pVelocity, int *pErrCode);
	BOOL		GetTargetVelocity(int nPhysAddr, int nSubIdx, int *pVelocity, int *pErrCode);
	BOOL		SetHomeMode(int nAxis, int HomeMethod, int *pErrCode);
	BOOL		SetHomeMode(int nPhysAddr, int nSubIdx, int HomeMethod, int *pErrCode);
	BOOL		GetHomeMode(int nAxis, int *pHomeMethod, int *pErrCode);
	BOOL		GetHomeMode(int nPhysAddr, int nSubIdx, int *pHomeMethod, int *pErrCode);
	BOOL		SetHomeSpeedEnv(int nAxis, int WorkSpeed, int StopSpeed, int AccDec, int *pErrCode);
	BOOL		SetHomeSpeedEnv(int nPhysAddr, int nSubIdx, int WorkSpeed, int StopSpeed, int AccDec, int *pErrCode);
	BOOL		GetHomeSpeedEnv(int nAxis, int *pWorkSpeed, int *pStopSpeed, int *pAccDec, int *pErrCode);
	BOOL		GetHomeSpeedEnv(int nPhysAddr, int nSubIdx, int *pWorkSpeed, int *pStopSpeed, int *pAccDec, int *pErrCode);
	BOOL		StResetPosition(int nAxis, int *pErrCode);
	BOOL		StResetPosition(int nPhysAddr, int nSubIdx, int *pErrCode);
	BOOL		StGetPosition(int nAxis, int TargCntr, int *pPosition, int *pErrCode);
	BOOL		StGetPosition(int nPhysAddr, int nSubIdx, int TargCntr, int *pPosition, int *pErrCode);
	BOOL		StGetCurSpeed(int nAxis, int TargCntr, int *pCurSpeed, int *pErrCode);
	BOOL		StGetCurSpeed(int nPhysAddr, int nSubIdx, int TargCntr, int *pCurSpeed, int *pErrCode);
	BOOL		StGetMotFlag(int nAxis, void *pMotFlag, int *pErrCode);
	BOOL		StGetMotFlag(int nPhysAddr, int nSubIdx, void *pMotFlag, int *pErrCode);
	BOOL		SetPolarity(int nAxis, int Polarity, int *pErrCode);
	BOOL		SetPolarity(int nPhysAddr, int nSubIdx, int Polarity, int *pErrCode);
	BOOL		GetPolarity(int nAxis, int *pPolarity, int *pErrCode);
	BOOL		GetPolarity(int nPhysAddr, int nSubIdx, int *pPolarity, int *pErrCode);
	BOOL		SetSoftLimit(int nAxis, int TargLimit, int PosLimit, int *pErrCode);
	BOOL		SetSoftLimit(int nPhysAddr, int nSubIdx, int TargLimit, int PosLimit, int *pErrCode);
	BOOL		GetSoftLimit(int nAxis, int TargLimit, int *pPosLimit, int *pErrCode);
	BOOL		GetSoftLimit(int nPhysAddr, int nSubIdx, int TargLimit, int *pPosLimit, int *pErrCode);
	BOOL		ReadMotState(int nAxis, int *pState, int *pErrCode);
	BOOL		ReadMotState(int nPhysAddr, int nSubIdx, int *pState, int *pErrCode);
	BOOL		SetWorkCurrent(int nAxis, int TargCurrent, int CurrValue, int *pErrCode);
	BOOL		SetWorkCurrent(int nPhysAddr, int nSubIdx, int TargCurrent, int CurrValue, int *pErrCode);
	BOOL		GetWorkCurrent(int nAxis, int TargCurrent, int *pCurrValue, int *pErrCode);
	BOOL		GetWorkCurrent(int nPhysAddr, int nSubIdx, int TargCurrent, int *pCurrValue, int *pErrCode);
	BOOL		SetStopCurrent(int nAxis, int CurrValue, int *pErrCode);
	BOOL		SetStopCurrent(int nPhysAddr, int nSubIdx, int CurrValue, int *pErrCode);
	BOOL		GetStopCurrent(int nAxis, int *pCurrValue, int *pErrCode);
	BOOL		GetStopCurrent(int nPhysAddr, int nSubIdx, int *pCurrValue, int *pErrCode);
	BOOL		SetStepDivision(int nAxis, int StepDiv, int *pErrCode);
	BOOL		SetStepDivision(int nPhysAddr, int nSubIdx, int StepDiv, int *pErrCode);
	BOOL		GetStepDivision(int nAxis, int *pStepDiv, int *pErrCode);
	BOOL		GetStepDivision(int nPhysAddr, int nSubIdx, int *pStepDiv, int *pErrCode);

	BOOL		SetSensorChan(int nPhysAddr, int nSubIdx, int nSensor, int SelectChan, int *pErrCode);
	BOOL		GetSensorChan(int nPhysAddr, int nSubIdx, int nSensor, int *pConnectChan, int *pErrCode);
	BOOL		SetSensorLogic(int nPhysAddr, int nSubIdx, int nSensor, int Logic, int *pErrCode);
	BOOL		GetSensorLogic(int nPhysAddr, int nSubIdx, int nSensor, int *pLogic, int *pErrCode);
	BOOL		ReadEncoderCnt(int nChannel, int TargCntr, int *pCountValue, int *pErrCode);
	BOOL		ReadEncoderCnt(int nPhysAddr, int nSubIdx, int TargCntr, int *pCountValue, int *pErrCode);
	BOOL		SetCntClear(int nChannel, int TargCntr, int *pErrCode);
	BOOL		SetCntClear(int nPhysAddr, int nSubIdx, int TargCntr, int *pErrCode);

	USHORT		DiGetNumChannel(void);
	USHORT		DoGetNumChannel(void);
	BOOL		DiGetOne(int DiChannel, int *pErrCode);
	BOOL		DiGetOne(int nPhysAddr, int DiChannel, int *pErrCode);
	USHORT		DiGetMulti(int IniChannel, int NumChannel, int *pErrCode);
	USHORT		DiGetMulti(int nPhysAddr, int *pErrCode);
	BOOL		DoGetOne(int DoChannel, int *pErrCode);
	BOOL		DoGetOne(int nPhysAddr, int DoChannel, int *pErrCode);
	USHORT		DoGetMulti(int IniChannel, int NumChannel, int *pErrCode);
	USHORT		DoGetMulti(int nPhysAddr, int *pErrCode);
	BOOL		DoPutOne(int DoChannel, BOOL OutState, int *pErrCode);
	BOOL		DoPutOne(int nPhysAddr, int DoChannel, BOOL OutState, int *pErrCode);
	BOOL		DoPutMulti(int IniChannel, int NumChannel, USHORT OutState, int *pErrCode);
	BOOL		DoPutMulti(int nPhysAddr, USHORT OutState, int *pErrCode);

	BOOL		ReadFwVersion(int nSlaveIdx, char *pVersion, int *pErrCode);
	BOOL		SetSettingSave(int nSlaveIdx, int *pErrCode);

	TCmc_AxisInPDO*		GetInPDO(int nAxis);
	TCmc_AxisInPDO*		GetInPDO(int nPhysAddr, int nSubIdx);
	TCmc_AxisOutPDO*	GetOutPDO(int nAxis);
	TCmc_AxisOutPDO*	GetOutPDO(int nPhysAddr, int nSubIdx);

	USHORT*				GetDiPDO(int nPhysAddr);
	USHORT*				GetDoPDO(int nPhysAddr);

};


typedef enum _DefCntr { 
	CNT_COMM, // Command Counter
	CNT_FEED, // Feedback Counter
}DefCntr;

// Flag: Bit  Axis ¸ ˸ ÷׵ 
typedef union{
	struct{
		t_ui16 RdyToSwOn: 		1; // Ready To Switch On/Off
		t_ui16 SwOn: 			1; // Switch On
		t_ui16 OperEnabled:		1; // Operation Enabled (Servo-on/off  ش)
		t_ui16 Fault:			1; // driver Fault ߻ 

		t_ui16 VoltEnabled:		1; // Voltage Enabled
		t_ui16 QuickStop: 		1; // Quick Stop
		t_ui16 SwOnDisabled:	1; // Switch On Disabled
		t_ui16 Warning:			1; // driver Warning ߻ 

		t_ui16 CtlrFault:		1; // Controller Fault ߻ 
		t_ui16 HomeError:		1; // Homing߿  ߻ߴ...
		t_ui16 OMS1:			1; // Operation Mode Specific 1
		t_ui16 IntLimit:		1; // Internal Limit Active (POT/NOT, Software Limit, Torque Limit, ...

		t_ui16 OMS2:			2; // Operation Mode Specific 2
		t_ui16 HomeBusy:		1; // Homing operation 
		t_ui16 HomeAttained:	1; // 1 - Homing   ϷǾ Ÿ.
	}bit;
	t_ui16 word;
}TSxSt_Flags;

typedef enum{
	DIR_P	= 0,
	DIR_N	= 1
}DefDir;

typedef enum{
	MIN_LIMIT	= 0,
	MAX_LIMIT	= 1
}DefSWLimit;

typedef enum{
	ST_STOP		= 0,
	ST_ACC		= 1,
	ST_DEC		= 2,
	ST_WORK		= 3
}DefMotStat;

typedef enum{
	RUN_CURR		= 0,
	ACC_CURR		= 1
}DefMotCurr;

typedef enum{
	STEP_FULL		= 0,
	STEP_HALF		= 1,
	STEP_1DIV4		= 2,
	STEP_1DIV8		= 3,
	STEP_1DIV16		= 4,
}DefStepDiv;

typedef enum{
	HOME_DEC		= 0,
	HOME_STOP		= 1,
	EL_NEG			= 2,
	EL_POS			= 3,
	SENSOR_POS		= 4,
}DefSensor;

typedef enum{
	LOGIC_A		= 0,
	LOGIC_B		= 1,
}DefLogic;

typedef enum{
	ENC_AB		= 0,
	ENC_Z		= 1,
}DefEncoder;

// ڵ  //
#define CMC_ERR_NONE						0
#define CMC_ERR_BASE						-700000
#define CMC_ERR_INVALID_SLAVE_ADDR			(CMC_ERR_BASE - 10) // Slave address setting   
#define CMC_ERR_INVALID_AL_STATE			(CMC_ERR_BASE - 20) // 
#define CMC_ERR_INVALID_OUTPDO_PTR			(CMC_ERR_BASE - 30) // 
#define CMC_ERR_INVALID_INPDO_PTR			(CMC_ERR_BASE - 31) // 
#define CMC_ERR_INVALID_PARAMETER			(CMC_ERR_BASE - 40) // Լ ߸ ĶͰ  Ǿ 
#define CMC_ERR_MASTER_INTERFACE_FAIL		(CMC_ERR_BASE - 200) // ̴Ĺ  ġ ͱȯ ߿   .
#define CMC_ERR_OPMODE_TIMEOUT				(CMC_ERR_BASE - 300) // Operation Mode ȯ  .

#pragma pack(pop)